#ifndef MYTH_MEDIA_MONITOR_H
#define MYTH_MEDIA_MONITOR_H

#include "mythmedia.h"
#include "mythcdrom.h"
#include <qvaluelist.h>
#include <qguardedptr.h>
#include <qthread.h>


const int kMediaEventType = 30042;

class MediaMonitor;

class MediaEvent : public QCustomEvent
{
 protected:
    MediaStatus m_OldStatus;
    QGuardedPtr<MythMediaDevice> m_Device;
 public:
    MediaEvent(MediaStatus oldStatus, MythMediaDevice* pDevice) : QCustomEvent(kMediaEventType) { m_OldStatus = oldStatus; m_Device = pDevice;}
    MediaStatus getOldStatus() const { return m_OldStatus; }
    MythMediaDevice* getDevice() { return m_Device; }
};

struct mntent;

class MonitorThread : public QThread
{
 protected:
    QGuardedPtr<MediaMonitor> m_Monitor;
    unsigned long m_Interval;
 public:
    MonitorThread( MediaMonitor* pMon,  unsigned long interval);
    void setMonitor(MediaMonitor* pMon) { m_Monitor = pMon; }
    virtual void run();
};

class MediaMonitor : public QObject
{
    Q_OBJECT

 protected:
    QValueList<MythMediaDevice*> m_Devices;
    bool m_Active;
    MonitorThread m_Thread;
    bool m_AllowEject;
 public:
    MediaMonitor(QObject* par, unsigned long interval, bool allowEject);
    bool addFSTab();
    void addDevice(MythMediaDevice* pDevice);
    bool addDevice(const char* dev);

    bool isActive() const { return m_Active; }
    void checkDevices();
    void startMonitoring();
    void stopMonitoring();

 public slots:
    void mediaStatusChanged( MediaStatus oldStatus, MythMediaDevice* pMedia);

};




#endif
